import sys
import json
import subprocess
import os

from signal import SIGTERM
from datetime import datetime

tweets = []
NER_DIR = '../../../twitter_nlp/'
os.environ['TWITTER_NLP'] = NER_DIR

WINDOW_SIZE = 300

curtime = -1
starttime = -1

def strip_tweet(t):
    filtered = []
    for w in t['tweet'].split():
        try:
            w.decode('utf-8')
            filtered.append(w)
        except UnicodeEncodeError:
            pass
    out = u'NONE\n'
    if filtered:
        out = u' '.join(filtered) + u'\n'
    return out

def concatenate_entities(msg):
    entities = {}
    current_token = ''
    current_type = ''
    for w in msg.split():
        tag = w.split('/')[-1]
        if tag[0] == 'B':
            if current_token and current_token != '':
                if current_type in entities:
                    entities[current_type].append(current_token)
                else:
                    entities[current_type] = [current_token]
            current_type = tag[2:]
            current_token = w[:-len(tag)-1]
        elif tag[0] == 'I':
            current_token += ' ' + w[:-len(tag)-1]
    if entities and current_token and current_token != '':
        if current_type in entities:
            entities[current_type].append(current_token)
        else:
            entities[current_type] = [current_token]
    
    return entities

def augment_tweets():
    ner = subprocess.Popen('python %spython/ner/extractEntities2.py --classify' % (NER_DIR),
                            shell=True,
                            close_fds=True,
                            stdin=subprocess.PIPE,
                            stdout=subprocess.PIPE)
    for t in tweets:
        txt = strip_tweet(t)
        ner.stdin.write(txt)

    ner.stdin.write('\n')

    for t in tweets:
        res = ner.stdout.readline().rstrip('\n')
        ent = concatenate_entities(res)
        if ent:
            t['entities'] = ent

    ner.stdin.close()
    ner.stdout.close()
    os.kill(ner.pid, SIGTERM)

def print_tweets():
    for i in tweets:
        sys.stdout.write(json.dumps(i) + u'\n')

for l in sys.stdin:
    if  curtime-starttime >= WINDOW_SIZE:
        sys.stderr.write('NERing ' + str(len(tweets)) + ' tweets...\n')
        augment_tweets()
        print_tweets()
        tweets = []
        starttime = curtime
    tweet = json.loads(l)
    curtime = int(tweet['timestamp']) / 1000
    if starttime < 0:
        starttime = curtime
    tweets.append(tweet)

sys.stderr.write('NERing ' + str(len(tweets)) + ' tweets...\n')
augment_tweets()
print_tweets()

