import sys, json, string, math

def getTokens(tw):
    tokens = []
    for t in tw.lower().split():
        if not 'http' in t:
            t = t.translate({ord(c): None for c in string.punctuation})
            if len(t) > 0:
                tokens.append(t)

    return tokens

def getEuclidNorm(vec):
    norm = 0
    for k,v in vec.iteritems():
        norm += math.pow(v, 2)

    return math.sqrt(norm)


def normalizeVector(vec):
    vec2 = {}
    norm = getEuclidNorm(vec)
    for k,v in vec.iteritems():
        if norm != 0:
            vec2[k] = vec[k] / norm
        else:
            vec2[k] = 0

    return vec2

def getVecsForTweets(tweets):
    uniqwords = {}
    posinmap = 0
    posmap = {}
    idfmap = {}
    tfs = {}
    vecs = {}
    num_docs = float(len(tweets))
    for k,v in tweets.iteritems():
        tfmap = {}
        words = getTokens(v)
        for w in words:
            if w in tfmap:
                tfmap[w] += 1
            else:
                tfmap[w] = 1
                if w in uniqwords:
                    uniqwords[w] += 1
                else:
                    uniqwords[w] = 1
                    posmap[w] = posinmap
                    posinmap += 1
        tfs[k] = tfmap
    for k,v in uniqwords.iteritems():
        idfmap[k] = math.log10(num_docs / v)
    for k,v in tfs.iteritems():
        vec = {}
        for ky,va in v.iteritems():
            vec[posmap[ky]] = va * idfmap[ky]
        vecs[k] = normalizeVector(vec)
    return vecs

def createCentroid(vecs):
    ctr = {}
    for key,val in vecs.iteritems():
        for k,v in val.iteritems():
            if k in ctr:
                ctr[k] += v
            else:
                ctr[k] = v

    size = len(vecs)
    for k,v in ctr.iteritems():
        ctr[k] = v/size
    return ctr

def computeCossim(v1, v2):
    dotProd = 0.0
    for k,v in v1.iteritems():
        if k in v2:
            dotProd += v*v2[k]

    norm = getEuclidNorm(v1) * getEuclidNorm(v2)
    cossim = 0.0

    if norm != 0:
        cossim = dotProd / norm

    return cossim

def getReprFromCluster(c):
    tweets = {}
    for e in c['tweets']:
        tweets[e[0]] = e[1]

    vecs = getVecsForTweets(tweets)
    ctr = createCentroid(vecs)

    bestC = -1.0
    bestT = None

    for k,v in vecs.iteritems():
        csim = computeCossim(v,ctr)
        if csim > bestC:
            bestC = csim
            bestT = (k,tweets[k])

    return bestT

def getMostRepr(s):
    clus = []

    for k,v in s['data'].iteritems():
        out = {}
        out[k] = {}
        out[k]['centroid'] = getReprFromCluster(v)[0]
        newID = 0
        oldID = 10e100
        entNum = 0
        newTw = None
        oldTw = None
        entTw = None
        for e in v['tweets']:
            if len(e) == 3 and e[2] > entNum:
                entTw = e[0]
                entNum = e[2]
            if e[0] > newID:
                newTw = e[0]
                newID = e[0]
            if e[0] < oldID:
                oldTw = e[0]
                oldID = e[0]
        out[k]['new'] = newTw
        out[k]['old'] = oldTw
        out[k]['ent'] = entTw
        clus.append(out)
    return clus

for l in sys.stdin:
    stories = json.loads(l)
    representative = getMostRepr(stories)
    for e in representative:
        sys.stdout.write(json.dumps(e) + '\n')
